package algae.gui;

import java.awt.*;
import java.awt.Graphics;
import java.awt.Graphics.*;
import java.awt.geom.*;
import java.awt.event.*;
import java.awt.image.*;

import java.awt.font.FontRenderContext;
import java.awt.font.TextLayout;
import java.awt.geom.AffineTransform;

import javax.swing.*;
import javax.swing.event.*;
import java.util.*;
import javax.swing.BorderFactory; 
import javax.swing.border.Border;
import javax.swing.border.TitledBorder;
import java.io.*;
import java.lang.Math.*;

import algae.simulation.WorldState;

public final class ImagePanel extends JPanel implements MouseListener, MouseMotionListener {

    MainGUI parent=null;

    static float scale = 5.0f;
    static float scaleHalf = scale/3.0f;

    int width,height;

    int x,y;
    
    Graphics2D big;
    BufferedImage bi;
    Rectangle area;

    boolean FirstTime=true;

    int pheroPos[][] = null;
    int substratePos[][] = null;
    int algaePos[][] = null;
    int lightPos[][] = null;
    int selected=-1;

    boolean doubleBuffer = true;
    boolean drawGrid = true;

    int mouseX=-1; 
    int mouseY=-1;
    
    int selX=-1;
    int selY=-1;

    public int getGridX()
    {
    	if (mouseX==-1) 
    	   return -1;
    	
    	if (mouseX>=(int)(width*scale))
    	   return -1;
    	
    	return (int)(mouseX/(int)scale);
    }
    
    public int getGridY()
    {
    	if (mouseY==-1) 
    	   return -1;
    	
    	if (mouseY>=(int)(height*scale))
    	   return -1;
    	
    	return (int)(mouseY/(int)scale);
    }    

    public int getSelX()
    {
    	if (mouseX==-1) 
    	   return -1;
    	    	
    	return selX;
    }
    
    public int getSelY()
    {
    	if (mouseY==-1) 
    	   return -1;
    	    	
    	return selY;
    }    

    public void mouseMoved(MouseEvent e) {
    	mouseX=e.getX();
    	mouseY=e.getY();
    	
    	int gx=getGridX();
        int gy=getGridY();
        /*
	JLabel mousePosLabel = new JLabel("Pos: --/--");
	JLabel mouseLightLabel = new JLabel("Light: ----");
	JLabel mouseSubstrateLabel = new JLabel("Substrate: ----");
	JLabel mousePheromoneLabel = new JLabel("Pheromone: ----");
	JLabel mouseAgentLabel = new JLabel("Agent: ----");
        */	
        if ((gx==-1)||(gy==-1))
            parent.mousePosLabel.setText("Pos: --/--");
        else
        {
            parent.mousePosLabel.setText("Pos: "+Integer.toString(gx)+"/"+Integer.toString(gy));
        	   
            int tmpInt=parent.contr.getAgentAt(gx,gy);
        	   
            if (tmpInt!=-1)
                parent.mouseAgentLabel.setText("Agent: "+Integer.toString(tmpInt));
            else
                parent.mouseAgentLabel.setText("Agent: ----");
     
            float tmpFloat=parent.contr.getSubstrateAt(gx,gy);           
            parent.mouseSubstrateLabel.setText("Substrate: "+Float.toString(tmpFloat));
            
            tmpFloat=parent.contr.getPheromoneAt(gx,gy);           
            parent.mousePheromoneLabel.setText("Pheromone: "+Float.toString(tmpFloat));

            tmpFloat=parent.contr.getLightAt(gx,gy);           
            parent.mouseLightLabel.setText("Light: "+Float.toString(tmpFloat));
        }
    }

    public void mousePressed(MouseEvent e) {
    }

    public void mouseDragged(MouseEvent e) {
    }

    public void mouseReleased(MouseEvent e) {
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    	mouseX=-1;
    	mouseY=-1;
    }

    public void mouseClicked(MouseEvent e) {
    	if ((getGridX()!=-1)&&(getGridY()!=-1))
    	{
	    selX=getGridX();
	    selY=getGridY();
	    
	    int tmpInt=parent.contr.getAgentAt(selX,selY);
	    if (tmpInt!=-1)
	    {
	    	parent.selection=tmpInt;
	    }
	    else
	    {
	    	parent.selection=-1;
	    }
	    
	    if (parent.selection!=-1)
	       parent.mouseSelectionLabel.setText("Selection: "+Integer.toString(parent.selection));
	    else
	       parent.mouseSelectionLabel.setText("Selection: ----");
	    
	    WorldState wsLocal=parent.contr.getState();
	    parent.paintAll(wsLocal,false);
    	}    	
    }

    public ImagePanel(MainGUI par) {
	x=0; y=0;
	if ((par.config.width>100)||(par.config.height>100))
	   doubleBuffer=false;
	else
	   doubleBuffer=true; 
	parent=par;
	
	addMouseMotionListener(this);
	addMouseListener(this);
    }

    public void setScale(float sc)
    {
    	scale=sc;
    	scaleHalf=sc/3.0f;
    	repaint();
    }

    public void clearPList()
    {
    	pheroPos=null;
    }
    
    public void clearAList()
    {
    	algaePos=null;
    }

    public void clearSList()
    {
    	substratePos=null;
    }
    
    public void clearLList()
    {
    	lightPos=null;
    }

    public void setPheromones(int pos[][]){
	pheroPos=pos;
    }

    public void setSubstrates(int pos[][]){
	substratePos=pos;
    }

    public void setLight(int pos[][]){
	lightPos=pos;
    }
	
    public void setAlgae(int sel, int pos[][]){
	selected=sel;
	algaePos=pos;
    }

    public void paintSubstrate(int pos[][]) {
	if (!parent.viewSubstrate())
	    return;
	if (pos==null)
	    return;
        for (int z=0; z<pos.length; z++)
	{
	    x=(int)((float)pos[z][0]*scale);
	    y=(int)((float)pos[z][1]*scale);
		
            int bright=pos[z][2]*10+5;
	    if (bright>255) bright=255;
	
	    big.setPaint(new Color(0,255,0,bright));
	    
	    big.fill(new Rectangle2D.Double(x,y,scale,scale));
	}
    } 

    public void paintLight(int pos[][]) {
	if (!parent.viewLight())
	    return;
	if (pos==null)
	    return;
        for (int z=0; z<pos.length; z++)
	{
	    x=(int)((float)pos[z][0]*scale);
	    y=(int)((float)pos[z][1]*scale);
		
            int bright=pos[z][2]*10+5;
	    if (bright>255) bright=255;
	
	    big.setPaint(new Color(255,255,0,bright));
	    
	    big.fill(new Rectangle2D.Double(x,y,scale,scale));
	}
    } 

    public void paintPheromone(int pos[][]) {
    	if (!parent.viewPheromone())
    	    return;
	if (pos==null)
	    return;
	for (int z=0; z<pos.length; z++)
	{
	    int x=(int)((float)pos[z][0]*scale);
	    int y=(int)((float)pos[z][1]*scale);
	    
	    int bright=pos[z][2]*5;
	    if (bright>255) bright=255;
	
	    big.setPaint(new Color(0,0,255,bright));
	    big.fill(new Rectangle2D.Double(x,y,scale,scale));
	}
    } 
    
    public void paintAlgae(int marked, int pos[][]) { 
    	if (!parent.viewAlgae())
    	    return;
	if (pos==null)
	    return;
	int c=0;
       for (int z=0; z<pos.length; z++)
       {
	if (z!=marked)
	{
            c=128-pos[z][3]/2;
	    big.setPaint(new Color(c,c,c));
	}
	else
	    big.setPaint(Color.red);	
	x=(int)(((float)pos[z][0]*scale)+(scale/2));
	y=(int)(((float)pos[z][1]*scale)+(scale/2));
	
	Polygon poly=new Polygon();
	
	switch (pos[z][2])
	{
	    case 1:
	        /*
	        big.drawLine((int)(x-scale/2),(int)(y-scale/2),(int)(x+scale/2),y);
		big.drawLine((int)(x+scale/2),y,(int)(x-scale/2),(int)(y+scale/2));
		big.drawLine((int)(x-scale/2),(int)(y-scale/2),(int)(x-scale/2),(int)(y+scale/2));
		*/
		poly.addPoint((int)(x-scale/2),(int)(y-scale/2));
		poly.addPoint((int)(x+scale/2),y);
		poly.addPoint((int)(x-scale/2),(int)(y+scale/2));
                break;
	    case 2:
	        /*
		big.drawLine((int)(x-scale/2),(int)(y-scale/2),x,(int)(y+scale/2));
		big.drawLine((int)(x-scale/2),(int)(y-scale/2),(int)(x+scale/2),(int)(y-scale/2));
		big.drawLine((int)(x+scale/2),(int)(y-scale/2),x,(int)(y+scale/2));
		*/
		poly.addPoint((int)(x-scale/2),(int)(y-scale/2));
		poly.addPoint(x,(int)(y+scale/2));
		poly.addPoint((int)(x+scale/2),(int)(y-scale/2));
               	break;
	    case 3:
	        /*
	   	big.drawLine((int)(x+scale/2),(int)(y-scale/2),(int)(x-scale/2),y);
		big.drawLine((int)(x-scale/2),y,(int)(x+scale/2),(int)(y+scale/2));
		big.drawLine((int)(x+scale/2),(int)(y-scale/2),(int)(x+scale/2),(int)(y+scale/2));
		*/
		poly.addPoint((int)(x+scale/2),(int)(y+scale/2));
		poly.addPoint((int)(x-scale/2),y);
		poly.addPoint((int)(x+scale/2),(int)(y-scale/2));
                break;
            case 4:
                /*
		big.drawLine((int)(x-scale/2),(int)(y+scale/2),x,(int)(y-scale/2));
		big.drawLine((int)(x-scale/2),(int)(y+scale/2),(int)(x+scale/2),(int)(y+scale/2));
		big.drawLine((int)(x+scale/2),(int)(y+scale/2),x,(int)(y-scale/2));
		*/
		poly.addPoint((int)(x-scale/2),(int)(y+scale/2));
		poly.addPoint(x,(int)(y-scale/2));
		poly.addPoint((int)(x+scale/2),(int)(y+scale/2));
               	break;
	}
	big.fill(poly);
     }
    }

    public void paintComponent(Graphics g)
    {
        super.paintComponent(g);
        //paintRefresh();
        this.update(g);
    }
  
    public void paintGrid()
    {
	if (!drawGrid)
	    return;
    	if (scale<=4.0f) // draw grid only if gridsize is big enough
    	    return;
    	
    	int zx=0;
    	int zy=0;
    	big.setColor(Color.gray);
    	big.drawRect(0,0,(int)((float)width*scale),(int)((float)height*scale));
    	
    	for (zx=0; zx<width; zx++)
    	     big.drawLine((int)((float)zx*scale),0,
    	                  (int)((float)zx*scale),(int)((float)height*scale));
    	for (zy=0; zy<height; zy++)
    	     big.drawLine(0,(int)((float)zy*scale),
    	                  (int)((float)width*scale),(int)((float)zy*scale));    	                  	
    }
    
    public void turnOnOffDB(boolean dbSwitch)
    {
	doubleBuffer=dbSwitch;
    }

    public void turnOnOffGrid(boolean gSwitch)
    {
	drawGrid=gSwitch;
	this.update(this.getGraphics());
    }

    public void setWidth(int w)
    {
    	width=w;
    }
    
    public void setHeight(int h)
    {
    	height=h;
    }
      
    public void paintAll(){
	paintPheromone(pheroPos);
	paintSubstrate(substratePos);
	paintLight(lightPos);
	paintAlgae(selected,algaePos);
    }

    public void update(Graphics g)
    {
	Graphics2D g2 = (Graphics2D)g;
	
	int w = (int)((float)width*scale);
	int h = (int)((float)height*scale);
	if (doubleBuffer)
	{
	    Dimension dim = getSize();
	    
	    if (FirstTime)
	    {
		area = new Rectangle(dim);
		bi = (BufferedImage)createImage(1000,1000);
		big = bi.createGraphics();
                 
		FirstTime = false;
	    }
    	    // Clears the rectangle that was previously drawn.
	    big.setColor(Color.white);
	    big.clearRect(0, 0, 1000,1000);
	} 
	else
	{
	    big=(Graphics2D)g;

	    // Clears the rectangle that was previously drawn.
	    big.setColor(Color.white);
	    big.clearRect(0, 0, w,h);
	}
 	
 	paintGrid();
 
        // Draws and fills the newly positioned rectangle to the buffer.
        paintAll();

        // Draws the buffered image to the screen.
	if (doubleBuffer)
	    g2.drawImage(bi, 0, 0, this);
    }
    
    public void paintRefresh()
    {
	repaint();
    }
}
