package algae.simulation;

import java.io.*;
import algae.simulation.Settings;
import algae.simulation.worlds.AgentWorld;
import algae.simulation.agents.Agent;
import java.lang.Math;

public class SNNSExport {
		
	public SNNSExport()
	{
	}
		
	public boolean exportANN(Agent a)
	{
		File snnsFile;
		String snnsFileName;
		FileOutputStream snnsStream;
		OutputStreamWriter snnsOStream;

		snnsFileName="agent"+a.agentID+".net";
		snnsFile=new File(snnsFileName);

                try {
	            snnsStream=new FileOutputStream(snnsFile);
	            snnsOStream = new OutputStreamWriter(snnsStream);
		} catch (IOException e) 
	        {
	            return false;
	        }
	        
	        try {
	        	
	            // write header of snns file
	            snnsOStream.write("SNNS network definition file V1.4-3D\n");
	            snnsOStream.write("generated at FRI AUG 18 23:09:29 2000\n");
	            snnsOStream.write("\n");
	            snnsOStream.write("network name : agent"+a.agentID+"\n");
		    snnsOStream.write("source files :\n");
		    snnsOStream.write("no. of units : "+/*a.phenoType.getActiveNeuronCount()*/a.phenoType.nCount+"\n");
		    snnsOStream.write("no. of connections : "+a.phenoType.getConnCount()+"\n");
		    snnsOStream.write("\n");
	            snnsOStream.flush();
	            
	            // write units section
	            snnsOStream.write("unit definition section :\n");
	            snnsOStream.write("\n");
	            snnsOStream.write("no. | typeName      | unitName | act      | bias     | st | position | act func | out func | sites\n");
		    snnsOStream.write("----|---------------|----------|----------|----------|----|----------|----------|----------|---------\n");
		    
		    int dummy=0;
		    
		    for (int z=0; z<a.phenoType.inCount; z++)
		    {
		    	String tmp=Integer.toString(z+1);
		    	while (tmp.length()<3) tmp=" "+tmp;
		    	snnsOStream.write(tmp+" |               |");
		    	
		    	if (z==0)
		    	   tmp=" energy";
		    	if ((z>=1)&&(z<5))
		    	{
		    	   dummy=z-1;
		    	   tmp=" bumper"+dummy;	
		    	}
		    	if ((z>=5)&&(z<9))
		    	{
		    	   dummy=z-5;
		    	   tmp=" phero"+dummy;	
		    	}
		    	if ((z>=9)&&(z<13))
		    	{
		    	   dummy=z-9;
		    	   tmp=" subst"+dummy;	
		    	}
		    	if ((z>=13)&&(z<17))
		    	{
		    	   dummy=z-13;
		    	   tmp=" light"+dummy;	
		    	}
		    	
		    	while (tmp.length()<10) tmp=tmp+" ";
		    	
		    	snnsOStream.write(tmp+"|");
		    	snnsOStream.write("  0.00000 |  0.00000 | i  |");
		    	
		    	tmp=" "+(z)+",1,0";
		    	while (tmp.length()<10) tmp=tmp+" ";
		    	snnsOStream.write(tmp+"|||\n");
		    			    	
		    	snnsOStream.flush();
	            }
	            
	            for (int z=a.phenoType.inCount; z<a.phenoType.inOutCount; z++)
		    {
		    	String tmp=Integer.toString(z+1);
		    	while (tmp.length()<3) tmp=" "+tmp;
		    	snnsOStream.write(tmp+" |               |");
		    	
		    	switch (z-a.phenoType.inCount)
		    	{
		    	     case 0: tmp=" stand";
		    	             break;
		    	     case 1: tmp=" right";
		    	             break;
		    	     case 2: tmp=" back";
		    	             break;
		    	     case 3: tmp=" left";
		    	             break;
		    	     case 4: tmp=" forward";
		    	}
		    	while (tmp.length()<10) tmp=tmp+" ";
		    	
		    	snnsOStream.write(tmp+"|");
		    	snnsOStream.write("  0.00000 |  0.00000 | o  |");
		    	
		    	tmp=" "+(z-a.phenoType.inCount+5)+",15,0";
		    	while (tmp.length()<10) tmp=tmp+" ";
		    	snnsOStream.write(tmp+"|||\n");
		    			    	
		    	snnsOStream.flush();
	            }
	           	           
	            int counter=a.phenoType.inOutCount;
	            for (int z=a.phenoType.inOutCount; z<a.phenoType.nCount; z++)
		    {
		    	//if (!a.phenoType.isActive(z)) continue;

		    	counter++;		    	
		    	
		    	String tmp=Integer.toString(z+1);
		    	while (tmp.length()<3) tmp=" "+tmp;
		    	snnsOStream.write(tmp+" |               |");
	    	
		    	tmp=" hid_"+(counter-a.phenoType.inOutCount-1);
		    	while (tmp.length()<10) tmp=tmp+" ";
		    	
		    	snnsOStream.write(tmp+"|");
		    	snnsOStream.write("  0.00000 |  0.00000 | h  |");
		    	
		    	int angle=z-a.phenoType.inOutCount;
		    	int angleStep=360/a.phenoType.hiddenCount/*getActiveNeuronCount()-a.phenoType.inOutCount*/;
		    	
		    	int x=6+(int)(5*Math.cos((double)(angle*angleStep)/180*Math.PI));
		    	int y=8+(int)(5*Math.sin((double)(angle*angleStep)/180*Math.PI));
		    	
		    	tmp=" "+x+","+y+",0";
		    	while (tmp.length()<10) tmp=tmp+" ";
		    	snnsOStream.write(tmp+"|||\n");
		    			    	
		    	snnsOStream.flush();
	            }
	            snnsOStream.write("----|---------------|----------|----------|----------|----|----------|----------|----------|---------\n");
	            snnsOStream.write("\n");
	                        	            
	            // write connection section
	            snnsOStream.write("connection definition section :\n");
	            snnsOStream.write("\n");
	            snnsOStream.write("target | site    | source:weight\n");
		    snnsOStream.write("-------|---------|------------------------------------------------------------------------------------------------------------\n");
		    
		    for (int z=a.phenoType.inCount; z<a.phenoType.nCount; z++)
		    {   
		    	if (!a.phenoType.isActive(z)) continue;
	    	
		    	int parentCount=a.phenoType.getParentNodeCount(z);
		    	
		    	if (parentCount>0)
		    	{
		    	    String tmp=Integer.toString(/*a.phenoType.getNth2Ac(z)*/z+1)+" ";
		    	    while (tmp.length()<7) tmp=" "+tmp;
		    	    
		    	    snnsOStream.write(tmp+"|         | ");
		            for (int j=0; j<parentCount; j++)
		    	    {
		    	    	int source=a.phenoType.getPrev(z,j+1);
		    	    	
		    	        snnsOStream.write(/*a.phenoType.getNth2Ac(source)*/(source+1)+": "+a.phenoType.weight_matrix[source][z]);
		    	        
		    	        if (j<parentCount-1)
		    	           snnsOStream.write(", ");			    	        
		    	    }		    	    
		    	    //System.out.println("");
		    	    snnsOStream.flush();
		    	    snnsOStream.write("\n");
			}		    	
		    }
		    snnsOStream.write("-------|---------|------------------------------------------------------------------------------------------------------------");
		    
		    snnsOStream.flush();
	            snnsStream.close();
	        } catch (IOException e)
	        {
	            System.out.println("Ouch, some error occured while saving agent"+a.agentID+".net");
	        }       
	        return true;  	
	}
}