package algae.simulation;

import java.io.*;
import algae.simulation.Settings;
import algae.simulation.worlds.AgentWorld;
import algae.simulation.agents.Agent;
import algae.simulation.SNNSExport;
import java.lang.Math;

public final class SimulationStats {
	
	File file;
	String fileName;
	FileOutputStream stream;
	OutputStreamWriter ostream;
	
	SNNSExport sExport=new SNNSExport();
	
	AgentWorld aw;
	Agent tmp;
	 	   
	Settings cfg;
	int annCnt=0;
	
	public SimulationStats(String s)
	{
	       fileName=s;
	       file=new File(s);
	}
	
	public boolean openStream(Settings set, AgentWorld agw)
	{
	       aw=agw;
	       cfg=set;
	       try {
	           stream=new FileOutputStream(file);
	           ostream = new OutputStreamWriter(stream);
	           
	           ostream.write("[SECTION SETTINGS]\n\n");
	           
	           tmp=(Agent)aw.agents.elementAt(0);
	           ostream.write("<SUBSECTION SIMULATION>\n");
	           ostream.write("steps = "+cfg.duration+"\n");
	           ostream.write("random_seed = "+cfg.randSeed+"\n");
	           ostream.write("world_width = "+cfg.width+"\n");
	           ostream.write("world_height = "+cfg.height+"\n");
	           ostream.write("neurons_used = "+tmp.phenoType.nCount+"\n");
	           ostream.write("input_neurons_used = "+tmp.phenoType.inCount+"\n");
	           ostream.write("output_neurons_used = "+tmp.phenoType.outCount+"\n");
	           ostream.write("initial_agents = "+cfg.initial_algaes+"\n");
	           ostream.write("initial_energy = "+cfg.initial_energy+"\n");
	           ostream.write("agent_transluency = "+cfg.agent_transluency+"\n");
	           ostream.write("pheromone_radius = "+cfg.pheromone_radius+"\n");
	           ostream.write("substrate = "+cfg.substrate_amount+"\n");
	           ostream.write("substrate_renew_rate = "+cfg.substrate_renew_rate+"\n");
	           ostream.write("pheromone_decrease_rate = "+cfg.pheromone_dec_rate+"\n");
                   ostream.write("pheromone_sensor_fov = "+cfg.pheromone_fov+"\n");
	           ostream.write("pheromone_sensor_range = "+cfg.pheromone_range+"\n");
	           ostream.write("substrate_sensor_fov = "+cfg.substrate_fov+"\n");
	           ostream.write("substrate_sensor_range = "+cfg.substrate_range+"\n");
	           ostream.write("light_sensor_fov = "+cfg.light_fov+"\n");
	           ostream.write("light_sensor_range = "+cfg.light_range+"\n");
	           ostream.write("bumper_sensor_range = "+cfg.bumper_range+"\n");         
	           ostream.write("energy_movement ="+cfg.energy_movement+"\n");
	           ostream.write("energy_mitose ="+cfg.energy_mitose+"\n");
	           ostream.write("energy_meiose ="+cfg.energy_meiose+"\n");
	           ostream.write("energy_pheromone_output ="+cfg.energy_pheromone_output+"\n");
       	           ostream.write("min_energy_mitose = "+cfg.min_energy_mitose+"\n");
    	           ostream.write("min_energy_meiose = "+cfg.min_energy_meiose+"\n");
    	           ostream.write("min_energy_pheromone_output = "+cfg.min_energy_pheromone_output+"\n");
                   ostream.write("max_pheromone_saturation = "+cfg.max_pheromone_saturation+"\n");
                   ostream.write("max_substrate_saturation = "+cfg.max_substrate_saturation+"\n");
    		   ostream.write("energy_substrate = "+cfg.energy_substrate+"\n");
    		   ostream.write("energy_light = "+cfg.energy_light+"\n");
                   ostream.write("\n");	      
	 	
	 	   for (int z=0; z<aw.agentCount; z++)
	 	   {
	 	   	tmp=(Agent)aw.agents.elementAt(z);
	 	   	
	 	   	ostream.write("<SUBSECTION AGENT #"+tmp.agentID+">\n");
	 	   	ostream.write("agent_index = "+z+"\n");

	 	   	ostream.write("position_x = "+tmp.x+"\n");
	 	   	ostream.write("position_y = "+tmp.y+"\n");
	 	   	ostream.write("energy_level = "+tmp.energy_level+"\n");
/*
	 	   	ostream.write("phenotype_miller_matrix\n");
	 	   		 	   	
	 	   	// stream out the phenotype  	 	   	
	 	   	for (int i=0; i<50; i++)
	 	   	{
	 	   	    for (int j=0; j<50; j++)
	 	   	    {
	 	   	    	ostream.write(tmp.phenoType.neuron_matrix[i][j]+" ");
	 	   	    }
	 	   	    ostream.write("\n");
	 	   	}
	 	   	
	 	   	ostream.write("phenotype_weight_matrix\n");
	 	   	
	 	   	for (int i=0; i<50; i++)
	 	   	{
	 	   	    for (int j=0; j<50; j++)
	 	   	    {
	 	   	    	ostream.write(tmp.phenoType.weight_matrix[i][j]+" ");
	 	   	    }
	 	   	    ostream.write("\n");
	 	   	}
	 	   	
	 	   	ostream.write("phenotype_spline_matrix\n");
	 	   	
	 	   	for (int i=0; i<2; i++)
	 	   	{
	 	   	    for (int j=0; j<10; j++)
	 	   	    {
	 	   	    	ostream.write(tmp.phenoType.spline_matrix[i][j]+" ");
	 	   	    }
	 	   	    ostream.write("\n");
	 	   	}	 	   		
*/	 	   	
	 	        ostream.write("\n");	      

	 	        ostream.flush();
	 	   }
	 	   //ostream.write("\n");	      
	 	
	 	   ostream.flush();
	           
	       } catch (IOException e) 
	       {
	       	   return false;
	       }
	       
	       return true;
	}
		
	public boolean writeStats(int stepNo)
	{
	       try {
                   ostream.write("[SECTION STEP "+stepNo+"]\n\n");
                   
                   ostream.write("<SUBSECTION GENERAL STATS>\n");
                   ostream.write("agents_alive = "+aw.aliveAgents()+"\n");
                   ostream.write("agents_died = "+aw.stepDeadCount+"\n");  
                   ostream.write("mitosis_count = "+aw.stepMitosisCount+"\n");  
                   ostream.write("meiosis_count = "+aw.stepMeiosisCount+"\n");  
                   ostream.write("light_intensity = "+aw.lw.intensity+"\n");
                   ostream.write("substrate_amount = "+aw.sw.totalAmount+"\n");
                   ostream.write("substrate_consumed = "+aw.sw.used+"\n");
                   ostream.write("substrate_renewed = "+aw.sw.renew+"\n");
                   ostream.write("pheromone_dissolved = "+aw.pw.used+"\n");
                   ostream.write("pheromone_renewed = "+aw.pw.renew+"\n");
                   ostream.write("light_consumed = "+aw.lw.used+"\n\n");
                        
                   for (int z=0; z<aw.agentCount; z++)
	 	   {
	 	   	tmp=(Agent)aw.agents.elementAt(z);
			
	 	   	if (tmp.died)
	 	   	{
                                //System.out.println("Exporting SNNS of dead agent");
	 	   		sExport.exportANN(tmp);
	 	   	 	//System.out.println("SIMULATIONSTATS.java: exporting dead agent");
	 	   		ostream.write("<SUBSECTION AGENT #"+tmp.agentID+">\n");
	 	   		ostream.write("alive = "+tmp.isAlive()+"\n");
	 	   		ostream.write("birth_step = "+tmp.birthStep+"\n");
	 	   		ostream.write("alife_steps = "+tmp.lifeTime+"\n");
	 	   		ostream.write("mitosis_count = "+tmp.gMitosisCount+"\n");
	 	   		ostream.write("meiosis_count = "+tmp.gMeiosisCount+"\n");
	 	   		ostream.write("used_light = "+tmp.gUsedLight+"\n");
	 	   		ostream.write("used_substrate = "+tmp.gUsedSubstrate+"\n");
				System.out.println("Checkpoint 1");
	 	   		if ((tmp.parent1==-1)&&(tmp.parent2==-1))
	 	   	    		ostream.write("number of parents = 0\n");
	 	   		else 
	 	   		if (tmp.parent2==-1)
	 	   		{
	 	   	    		ostream.write("number of parents = 1\n");
	 	   	    		ostream.write("parent_1 = "+tmp.parent1+"\n");
	 	   		}
	 	   		else 
	 	   		{
	 	   	    		ostream.write("number_of_parents = 2\n");
	 	   	    		ostream.write("parent_1 = "+tmp.parent1+"\n");
	 	   	    		ostream.write("parent_2 = "+tmp.parent2+"\n");
	 	   		}	
	 	   	 	   	
				System.out.println("Checkpoint 2");
				ostream.write("number_of_children = "+tmp.childCount+"\n");
				if (tmp.childCount>0)
				{
			    		ostream.write("children_list\n");
			    		int cList[]=tmp.getChildList();
			    		for (int i=0; i<tmp.childCount; i++)
			    		{
			        		ostream.write(cList[i]+" ");
			    		}
			    		ostream.write("\n");
				}

				System.out.println("Checkpoint 2");
				// write the following matrices only in the summary
				// to reduce space waste
	 	   		ostream.write("phenotype_miller_matrix\n");
	 	   		 	   	
	 	   		// stream out the phenotype  	 	   	
	 	   		for (int i=0; i<tmp.phenoType.nCount; i++)
	 	   		{
	 	   	    		for (int j=0; j<tmp.phenoType.nCount; j++)
	 	   	    		{
	 	   	    			if (tmp.phenoType.neuron_matrix[i][j]!=0)
	 	   	    	   			ostream.write("1 ");
	 	   	    			else
	 	   	    	   			ostream.write("0 ");
	 	   	    		}
	 	   	    		ostream.write("\n");
	 	   		}
				System.out.println("Checkpoint 3");
	 	   		ostream.write("phenotype_weight_matrix\n");
				
	 	   		for (int i=0; i<tmp.phenoType.nCount; i++)
	 	   		{
	 	   	    		for (int j=0; j<tmp.phenoType.nCount; j++)
	 	   	    		{
	 	   	    			ostream.write(tmp.phenoType.weight_matrix[i][j]+" ");
	 	   	    		}
	 	   	    		ostream.write("\n");
	 	   		}
	 	   	
				System.out.println("Checkpoint 4");

				if (tmp.cfg.spline_node_count!=-1)
				{
				    ostream.write("phenotype_spline_matrix\n");
				    for (int i=0; i<2; i++)
				    {	
	 	   	    		for (int j=0; j<tmp.cfg.spline_node_count; j++)
	 	   	    		{
	 	   	    			ostream.write(tmp.phenoType.spline_matrix[i][j]+" ");
	 	   	    		}
	 	   	    		ostream.write("\n");
				    }
	 	   		}	 	   		

	 	        	ostream.write("\n");	      
	 	        	ostream.flush();
				System.out.println("Checkpoint 5");
	 	        	continue;
	 	   	}
	 	   	
			//System.out.println("SIMULATIONSTATS.java: exporting alive agent");
	 	   	ostream.write("<SUBSECTION AGENT #"+tmp.agentID+">\n");
	 	   	ostream.write("agent_index = "+z+"\n");
	 	   	ostream.write("heading = "+tmp.dir+"\n");
	 	   	ostream.write("position_x = "+tmp.x+"\n");
	 	   	ostream.write("position_y = "+tmp.y+"\n");
	 	   	ostream.write("energy_level = "+tmp.energy_level+"\n");
	 	   	ostream.write("ann_output = "+tmp.ann_output+"\n");
	 	   	ostream.write("mitosis = "+tmp.madeMitosis+"\n");
	 	   	ostream.write("meiosis = "+tmp.madeMeiosis+"\n");
	 	   	ostream.write("used_light = "+tmp.usedLight+"\n");
	 	   	ostream.write("used_substrate = "+tmp.usedSubstrate+"\n");
	 	   	//ostream.write("died = "+tmp.died+"\n");
	 	   	ostream.write("birth = "+tmp.born+"\n");
	 	   	ostream.write("sensors_values\n");
	 	   	
	 	   	for (int i=0; i<tmp.SENSORCOUNT; i++)
	 	   	{
	 	   	     ostream.write(tmp.sensor_values[i]+" ");
	 	   	}
	 	   	ostream.write("\n");
	 	   	/*   	
	 	   	ostream.write("phenotype_miller_matrix\n");
	 	   	
	 	   	// stream out the phenotype
	 	   	for (int i=0; i<50; i++)
	 	   	{
	 	   	    for (int j=0; j<50; j++)
	 	   	    {
	 	   	    	ostream.write(tmp.phenoType.neuron_matrix[i][j]+" ");
	 	   	    }
	 	   	    ostream.write("\n");
	 	   	}
	 	   	
	 	   	ostream.write("phenotype_weight_matrix\n");
	 	   	
	 	   	for (int i=0; i<50; i++)
	 	   	{
	 	   	    for (int j=0; j<50; j++)
	 	   	    {
	 	   	    	ostream.write(tmp.phenoType.weight_matrix[i][j]+" ");
	 	   	    }
	 	   	    ostream.write("\n");
	 	   	}
	 	   	
	 	   	ostream.write("phenotype_spline_matrix\n");
	 	   	
	 	   	for (int i=0; i<2; i++)
	 	   	{
	 	   	    for (int j=0; j<10; j++)
	 	   	    {
	 	   	    	ostream.write(tmp.phenoType.spline_matrix[i][j]+" ");
	 	   	    }
	 	   	    ostream.write("\n");
	 	   	}	 	   		
	 	   	*/
	 	        ostream.write("\n");	      
	 	        ostream.flush();
	 	   }
	 	   //ostream.write("\n");	      
	 	                  
                   ostream.flush();
	           
	       } catch (IOException e)
	       {
	       	     return false;
	       }
	       return true;
	}
	
	public boolean writeSummary()
	{
	       try {
                   ostream.write("[SECTION SUMMARY]\n\n");
                   
                   ostream.write("<SUBSECTION GENERAL STATS>\n");
                   //ostream.write("dead_agents = "+aw.deadAgents()+"\n");
                   ostream.write("living_agents = "+aw.livingAgents()+"\n\n");  
                   
                   for (int z=0; z<aw.agentCount; z++)
	 	   {
	 	   	tmp=(Agent)aw.agents.elementAt(z);
	 	   
	 	   	if (tmp.died)
	 	   	   continue;
	 	   	   
	 	        sExport.exportANN(tmp);
	 	   	 	   	
	 	   	ostream.write("<SUBSECTION AGENT #"+tmp.agentID+">\n");
	 	   	ostream.write("alive = "+tmp.isAlive()+"\n");
	 	   	ostream.write("birth_step = "+tmp.birthStep+"\n");
	 	   	ostream.write("alife_steps = "+tmp.lifeTime+"\n");
	 	   	ostream.write("mitosis_count = "+tmp.gMitosisCount+"\n");
	 	   	ostream.write("meiosis_count = "+tmp.gMeiosisCount+"\n");
	 	   	ostream.write("used_light = "+tmp.gUsedLight+"\n");
	 	   	ostream.write("used_substrate = "+tmp.gUsedSubstrate+"\n");
	 	   	if ((tmp.parent1==-1)&&(tmp.parent2==-1))
	 	   	    ostream.write("number of parents = 0\n");
	 	   	else 
	 	   	if (tmp.parent2==-1)
	 	   	{
	 	   	    ostream.write("number of parents = 1\n");
	 	   	    ostream.write("parent_1 = "+tmp.parent1+"\n");
	 	   	}
	 	   	else 
	 	   	{
	 	   	    ostream.write("number_of_parents = 2\n");
	 	   	    ostream.write("parent_1 = "+tmp.parent1+"\n");
	 	   	    ostream.write("parent_2 = "+tmp.parent2+"\n");
	 	   	}
	 	   	 	   	
			ostream.write("number_of_children = "+tmp.childCount+"\n");
			if (tmp.childCount>0)
			{
			    ostream.write("children_list\n");
			    int cList[]=tmp.getChildList();
			    for (int i=0; i<tmp.childCount; i++)
			    {
			        ostream.write(cList[i]+" ");
			    }
			    ostream.write("\n");
			}

			// write the following matrices only in the summary
			// to reduce space waste
	 	   	ostream.write("phenotype_miller_matrix\n");
	 	   		 	   	
	 	   	// stream out the phenotype  	 	   	
	 	   	for (int i=0; i<tmp.phenoType.nCount; i++)
	 	   	{
	 	   	    for (int j=0; j<tmp.phenoType.nCount; j++)
	 	   	    {
	 	   	    	if (tmp.phenoType.neuron_matrix[i][j]!=0)
	 	   	    	   ostream.write("1 ");
	 	   	    	else
	 	   	    	   ostream.write("0 ");
	 	   	    }
	 	   	    ostream.write("\n");
	 	   	}
	 	   	
	 	   	ostream.write("phenotype_weight_matrix\n");
	 	   	
	 	   	for (int i=0; i<tmp.phenoType.nCount; i++)
	 	   	{
	 	   	    for (int j=0; j<tmp.phenoType.nCount; j++)
	 	   	    {
	 	   	    	ostream.write(tmp.phenoType.weight_matrix[i][j]+" ");
	 	   	    }
	 	   	    ostream.write("\n");
	 	   	}

	 	   	if (tmp.cfg.spline_node_count!=-1)
			{
			    ostream.write("phenotype_spline_matrix\n");
	 	   	
			    for (int i=0; i<2; i++)
			    {
				for (int j=0; j<10; j++)
				{
				    ostream.write(tmp.phenoType.spline_matrix[i][j]+" ");
				}
				ostream.write("\n");
			    }
	 	   	}	 	   		

	 	        ostream.write("\n");	      
	 	        ostream.flush();
	 	   }
	 	   ostream.write("\n");	      
	 	                  
                   ostream.flush();
	           
	       } catch (IOException e)
	       {
	       	     return false;
	       }
	       return true;
	}
	
	public boolean closeStream()
	{
	       try {
	       	   ostream.flush();
	           stream.close();
	       } catch (IOException e)
	       {
	       	   return false;
	       }
	       return true;
	}
}
