package algae.simulation.agents;

import netView.NetViewSource; //changed by cb
import java.util.*;
import java.util.Random;
import algae.simulation.Settings;
import algae.simulation.worlds.PheromoneWorld;
import algae.simulation.worlds.AgentWorld;
import algae.simulation.worlds.SubstrateWorld;
import algae.simulation.worlds.LightWorld;

/** A class that represents ...
 * 
 * @see OtherClasses
 * @author your_name_here
 */
public class Agent implements netView.NetViewSource {

    public final static int SENSORCOUNT = 16;

    public GenoType genoType;
    public PhenoType phenoType;

    public AgentSensor sensors[]=new AgentSensor[4];
    
    public PheromoneWorld pheromoneWorld;
    public SubstrateWorld substrateWorld;
    public AgentWorld agentWorld;
    public LightWorld lightWorld;
    
    public int x;
    public int y;
    public int dir;
    
    public int agentID=0;
    
    // for statistics
    public int birthStep = 0;
    public int lifeTime = 0;
    public int gMitosisCount = 0;
    public int gMeiosisCount = 0;
    public float gUsedLight = 0.0f;
    public float gUsedSubstrate = 0.0f;    

    // parent1 
    public int parent1 = -1;
    public int parent2 = -1;
    
    public Vector children = new Vector(5,1);
    public int childCount = 0;
    
    public boolean dead=false;

    public float energy_level = 0.0f;

    public int ann_output=0;
    public float [] sensor_values = new float[SENSORCOUNT];

    public boolean usedSubstrate = false;
    public boolean usedLight = false;
    public boolean madeMitosis = false;
    public boolean madeMeiosis = false;
    public boolean died = false;
    public boolean born = false;
    
    public Settings cfg;
        
    final static float mRate = 0.1f;
 
    public String[] getInNeuronsDesc()
    {
 	String []inDesc={"energy",
 	                 "bumper0","bumper1","bumper2","bumper3",
 	                 "phero0","phero1","phero2","phero3",
 	                 "subst0","subst1","subst2","subst3",
 	                 "light0","light1","light2","light3"};
	return inDesc;
    }

    public String[] getOutNeuronsDesc()
    {
 	String []outDesc={"stand","right","back","left","forward"};   	
   	return outDesc;
    }
    
    public byte[][] getTopology()
    {
    	return phenoType.neuron_matrix;
    }

    public float[][] getWeights()
    {
    	return phenoType.weight_matrix;
    }

    public int[] getInputNodeIndices()
    {
    	int retVal[]=new int[phenoType.inCount];
    	for (int i=0; i<phenoType.inCount; i++)
	    retVal[i]=i;
	    
	return retVal;	
    }

    public float[] getNetActivations()
    {    
	return phenoType.neuronOutput;	
    }

    public boolean running() //added by cb
    {
	// are there values for the neuron actiavtion
	return(true);
    }
     public int getID() //added by cb
	 {
	     return agentID;
	 }

    public int[] getOutputNodeIndices()
    {
    	int retVal[]=new int[phenoType.outCount];
    	for (int i=phenoType.inCount+phenoType.hiddenCount; i<phenoType.nCount; i++)
	    retVal[i-phenoType.inCount-phenoType.hiddenCount]=i;
	    
	return retVal;	
    }

    public boolean isAlive()
    {
    	return (!(dead | died));
    }

    // randomly initialized agent
    public Agent(int sx, int sy, Settings c, Random rand,
                 AgentWorld aw, SubstrateWorld sw,
                 PheromoneWorld pw, LightWorld lw, int aID)
    {
	cfg=c;
	x=sx;
	y=sy;
	agentID=aID;
	dir=Math.abs(rand.nextInt()%4)+1; 
		
	agentWorld=aw;
	lightWorld=lw;
	substrateWorld=sw;
	pheromoneWorld=pw;
	
	energy_level=cfg.initial_energy;

        genoType = new GenoType(rand.nextInt(),cfg);
	phenoType = new PhenoType(genoType);
	
	// bumper sensor
	sensors[0]=new AgentSensor(cfg,cfg.bumper_range,10);
	sensors[0].setLut(agentWorld.lut);
	// pheromone sensor
	sensors[1]=new AgentSensor(cfg,cfg.pheromone_range,cfg.pheromone_fov);
	sensors[1].setLut(agentWorld.lut);
	// substrate sensor
	sensors[2]=new AgentSensor(cfg,cfg.substrate_range,cfg.substrate_fov);
	sensors[2].setLut(agentWorld.lut);
	// light sensor
	sensors[3]=new AgentSensor(cfg,cfg.light_range,cfg.light_fov);
	sensors[3].setLut(agentWorld.lut);
    }

    // meiosis
    public Agent(Agent a, Agent b, Settings c, Random rand, int aID)
    {
	cfg=c;
	agentID=aID;
	dir=Math.abs(rand.nextInt()%4)+1; 
	
	agentWorld=a.agentWorld;
	lightWorld=a.lightWorld;
	substrateWorld=a.substrateWorld;
	pheromoneWorld=a.pheromoneWorld;
		
	b.energy_level=b.energy_level-cfg.energy_mitose;
	a.energy_level=a.energy_level-cfg.energy_mitose;
	energy_level=cfg.initial_energy;

	genoType = new GenoType(cfg.randSeed, a.genoType, b.genoType, cfg);
	phenoType = new PhenoType(genoType);
	
	// bumper sensor
	sensors[0]=new AgentSensor(cfg,cfg.bumper_range,10);
	sensors[0].setLut(agentWorld.lut);
	// pheromone sensor
	sensors[1]=new AgentSensor(cfg,cfg.pheromone_range,cfg.pheromone_fov);
	sensors[1].setLut(agentWorld.lut);
	// substrate sensor
	sensors[2]=new AgentSensor(cfg,cfg.substrate_range,cfg.substrate_fov);
	sensors[2].setLut(agentWorld.lut);
	// light sensor
	sensors[3]=new AgentSensor(cfg,cfg.light_range,cfg.light_fov);
	sensors[3].setLut(agentWorld.lut);
    }

    // mitosis
    public Agent(Agent a, Settings c, Random rand, int aID)
    {
	cfg=c;
	agentID=aID;
	dir=Math.abs(rand.nextInt()%4)+1; 
		
	agentWorld=a.agentWorld;
	lightWorld=a.lightWorld;
	substrateWorld=a.substrateWorld;
	pheromoneWorld=a.pheromoneWorld;
		
	a.energy_level=a.energy_level-cfg.energy_meiose;
	energy_level=cfg.initial_energy;

	genoType = new GenoType(cfg.randSeed, a.genoType, cfg);
	phenoType = new PhenoType(genoType);
	
	// bumper sensor
	sensors[0]=new AgentSensor(cfg,cfg.bumper_range,10);
	sensors[0].setLut(agentWorld.lut);
	// pheromone sensor
	sensors[1]=new AgentSensor(cfg,cfg.pheromone_range,cfg.pheromone_fov);
	sensors[1].setLut(agentWorld.lut);
	// substrate sensor
	sensors[2]=new AgentSensor(cfg,cfg.substrate_range,cfg.substrate_fov);
	sensors[2].setLut(agentWorld.lut);
	// light sensor
	sensors[3]=new AgentSensor(cfg,cfg.light_range,cfg.light_fov);
	sensors[3].setLut(agentWorld.lut);
    }
    
    public float[] getInfos()
    {
    	float invec[] = new float[19];
	
	float tmp[] = new float[4];

	invec[0]=(float)this.dir;
	invec[1]=this.energy_level;
	
	// get light sensor values
	tmp=sensors[3].getValuesL(dir,x,y,lightWorld);
	for (int z=0; z<4; z++)
	    invec[z+2]=(float)tmp[z];
	
	// get substrate sensor values
	tmp=sensors[2].getValuesS(dir,x,y,substrateWorld);
	for (int z=0; z<4; z++)
	    invec[1*4+z+2]=(float)tmp[z];
	
	// get pheromone sensor values
	tmp=sensors[1].getValuesP(dir,x,y,pheromoneWorld);
	for (int z=0; z<4; z++)
	    invec[2*4+z+2]=(float)tmp[z];
	
	// get bumper sensor values
	tmp=sensors[0].getValuesA(dir,x,y,agentWorld);
	for (int z=0; z<4; z++)
	    invec[3*4+z+2]=(float)tmp[z];
		
        invec[18]=(float)ann_output;
	
	// apply sensitivity
	for (int z=0; z<4; z++) // apply for light
	    invec[z+2]*=phenoType.sensitivity[3];
	    
        for (int z=0; z<4; z++) // apply for light
	    invec[1*4+z+2]*=phenoType.sensitivity[2];
	    
        for (int z=0; z<4; z++) // apply for light
	    invec[2*4+z+2]*=phenoType.sensitivity[1];
	    
        for (int z=0; z<4; z++) // apply for light
	    invec[3*4+z+2]*=phenoType.sensitivity[0];
        		
	return invec;
    }

    // if the agent made mitosis or was involved in meiosis
    // the child is added to the child-list
    public void addChild(int i)
    {
    	children.add(new Integer(i));
    	childCount++;
    }

    // return the list of all children
    public int[] getChildList()
    {
    	if (childCount==0)
    	   return null; // no children, return null
    	
    	int retVal[] = new int[childCount];
    	
    	for (int z=0; z<childCount; z++)
    	{
    	    Integer tmp=(Integer)(children.elementAt(z));
    	    retVal[z]=tmp.intValue();
    	}
    	return retVal;
    }
    
    public void setParents(int p1, int p2)
    {
    	parent1=p1;
    	parent2=p2; // this parent is only used for meiosis
    }

    // calculates a step (neural network) and returns the winner neuron
    public int calcStep()
    {
	float invec[] = new float[17];
	invec[0]=energy_level;

	float tmp[] = new float[4];

	// get bumper sensor values
	tmp=sensors[0].getValuesA(dir,x,y,agentWorld);
	for (int z=0; z<4; z++)
	    invec[z+1]=(float)tmp[z];
	
	// get pheromone sensor values
	tmp=sensors[1].getValuesP(dir,x,y,pheromoneWorld);
	for (int z=0; z<4; z++)
	    invec[4+z+1]=(float)tmp[z];
		
	// get substrate sensor values
	tmp=sensors[2].getValuesS(dir,x,y,substrateWorld);
	for (int z=0; z<4; z++)
	    invec[2*4+z+1]=(float)tmp[z];
	
	// get light sensor values
	tmp=sensors[3].getValuesL(dir,x,y,lightWorld);
	for (int z=0; z<4; z++)
	    invec[3*4+z+1]=(float)tmp[z];
	
	// transfer all sensor values to a temporary storage for later use (statistics)
	for (int z=0; z<SENSORCOUNT; z++)
	{
	    sensor_values[z]=invec[1+z];	
	}
	
	ann_output=phenoType.processNet(invec);	
	return ann_output;
    }
    
} /* end class Agent */
