package algae.simulation.agents;

import java.util.*;
import java.lang.Object;
import java.lang.*;
import java.util.BitSet;
import java.util.Random;
import java.lang.Math;
import algae.simulation.Settings;

public class GenoType {

    public int nCount = 50; // count of neurons used
    
    public int inCount = 17; // number of input-neurons
    public int outCount = 5; // number of output-neurons

    public int inOutCount = inCount+outCount;
    public int hiddenCount = nCount-inOutCount;

    public int splineNodeCount = -1;

    public int funcSel = -1;
    public float genoANNBias=0.0f;

    // the DNA string
    BitSet nm_bit;  // 50x50 "booleans" 
    BitSet wm_bit; // 50x50 32-bit IEEE floats
    BitSet sm_bit;   // 10x2 32-bit IEEE floats
    BitSet ss_bit;   // sensitivity bitstring (fr jeden sensor: 4)
    BitSet mm_bit;   // metabolic markers
    BitSet csnActive_bit; // shows wether a specific node of the cubic splines is active
           
    Random rand;
    
    int crippled = 0;
    int randSeed=0;
    Settings c;
    
    // constructor for randomly initialized agent
    public GenoType(int rs, Settings cfg)
    {
	c=cfg;
    	nCount=cfg.number_of_neurons;
        hiddenCount = nCount-inOutCount;
	splineNodeCount = cfg.spline_node_count;
	genoANNBias=cfg.initial_ann_bias;
		
	funcSel=cfg.ann_transfer_func;

    	randSeed=rs;
    	rand=new Random(randSeed);

	nm_bit = new BitSet(nCount*nCount);
	wm_bit = new BitSet(nCount*nCount*32);
		
	if (splineNodeCount>0)
	{
	   sm_bit = new BitSet(64*splineNodeCount*nCount);
	   csnActive_bit = new BitSet(splineNodeCount*nCount);
    	   initRandomSm();    	
    	   initRandomCsn();
	}

	ss_bit=new BitSet(4);
	mm_bit=new BitSet(2);
	
	initRandomMM();
	initRandomSS();
	initRandomNm();
    	initRandomWm();
    }

    public boolean isCsnBit(int i)
    {
    	return csnActive_bit.get(i);
    }

    public int getCsnCount(int neuronIdx)
    {
    	int tmp=0;
    	for (int z=0; z<splineNodeCount; z++)
    	{
    	    if (csnActive_bit.get(neuronIdx*splineNodeCount+z)==true)
    	    	tmp++;
    	}
    	return tmp;
    }
    
    // constructor for cross over agent (meiosis)
    public GenoType(int rs, GenoType a, GenoType b, Settings cfg)
    {
	c=cfg;
    	nCount=cfg.number_of_neurons;
        hiddenCount = nCount-inOutCount;
	splineNodeCount = cfg.spline_node_count;
	
	//TODO: implement a better mutation function for bias
	genoANNBias=(a.genoANNBias+b.genoANNBias)/2;
	
	funcSel=cfg.ann_transfer_func;

    	randSeed=rs;
    	rand=new Random(randSeed);

	nm_bit = new BitSet(nCount*nCount);
	wm_bit = new BitSet(nCount*nCount*32);
	ss_bit = new BitSet(4);
	mm_bit=new BitSet(2);
	
	if (splineNodeCount>0)
	{
	   sm_bit = new BitSet(64*splineNodeCount*nCount);
	   csnActive_bit = new BitSet(splineNodeCount*nCount);
	   initRandomCsn();
	}

    	this.crossOver(a,b);
    	this.mutate();
    	if (splineNodeCount>0)
	    correctCsn();
    }
    
    // constructor for mutation (mitosis)
    public GenoType(int rs, GenoType a, Settings cfg)
    {
	c=cfg; 

	randSeed=rs;

	rand=new Random(randSeed);
       
	nCount=cfg.number_of_neurons;
        hiddenCount = nCount-inOutCount;
	splineNodeCount = cfg.spline_node_count;
	//TODO: implement a better mutation function for bias
	genoANNBias=(a.genoANNBias+Math.abs(rand.nextFloat()))/2;
	
	funcSel=cfg.ann_transfer_func;

	nm_bit = new BitSet(nCount*nCount);
	wm_bit = new BitSet(nCount*nCount*32);
	if (splineNodeCount>0)
	{
	   sm_bit = new BitSet(64*splineNodeCount*nCount);
	   csnActive_bit = new BitSet(splineNodeCount*nCount);
	   this.csnActive_bit = a.csnActive_bit;
    	   this.sm_bit=a.sm_bit;
	}
    	ss_bit = new BitSet(4);

	this.mm_bit=a.mm_bit;
    	this.ss_bit=a.ss_bit;
    	this.nm_bit=a.nm_bit;
    	this.wm_bit=a.wm_bit;
	this.mutate();
	if (splineNodeCount>0)
	    correctCsn();
    }
    
    // return the crippleness of an agent (after mutation and/or crossover)
    public int crippleness()
    {
	return crippled;
    }

    // return if agent is crippled
    public boolean isCrippled()
    {
    	return (crippled>0);
    }

    //TODO: implement overcrossing and mutation of csn_active...
    
    // make a new genotype out of two others (meiosis)
    public void crossOver(GenoType a, GenoType b)    
    {
	int crossingPos = Math.abs(rand.nextInt()) % nm_bit.size();
    	int cop=0; // actual number of crossover-point
    	int cs=0;
    	
    	// crossing over nm_bit
    	for (int z=0; z<a.nm_bit.size(); z++)
    	{
    	    if ((cop<c.crossover_type)&&(Math.abs(rand.nextFloat())<c.ann_crossover_rate))
	    {
		cop++;
    	    	cs=1-cs;
	    }
    		
    	    if (cs==0)
    	    {
    	    	if (b.nm_bit.get(z)==true)
    	    	    nm_bit.set(z);
    	    	else
    	    	    nm_bit.clear(z);
    	    }
    	    else
    	    {
    	    	if (a.nm_bit.get(z)==true)
    	    	    nm_bit.set(z);
    	    	else
    	    	    nm_bit.clear(z);
    	    }
    	}

	cop=0;
    	// crossing over csnActive_bit
	if (c.spline_node_count!=-1)
    	for (int z=0; z<a.csnActive_bit.size(); z++)
    	{
    	    if ((cop<c.crossover_type)&&(Math.abs(rand.nextFloat())<c.cs_crossover_rate))
	    {
		cop++;
    	    	cs=1-cs;
	    }
    		
    	    if (cs==0)
    	    {
    	    	if (b.csnActive_bit.get(z)==true)
    	    	    csnActive_bit.set(z);
    	    	else
    	    	    csnActive_bit.clear(z);
    	    }
    	    else
    	    {
    	    	if (a.csnActive_bit.get(z)==true)
    	    	    csnActive_bit.set(z);
    	    	else
    	    	    csnActive_bit.clear(z);
    	    }
    	}
    	
    	// validate the nm_bit matrix
	for (int i=0; i<nCount; i++)
	    for (int j=0; j<nCount; j++)
	    {
	    	if ((i>j)&&(nm_bit.get(j*nCount+i)==true))
	    	{
	    	   if ((i>=inOutCount && j<inCount)||    // hidden -> input
	    	       (i>=inCount && i<inOutCount)||   // output -> ???
	    	       (j<inCount && i<inCount)||       // input -> input
	    	       (i>=inOutCount && j>=inOutCount))	// hidden -> hidden (src>target)   	        
	    	       
	    	      nm_bit.clear(j*nCount+i);
	    	}
	    	if ((i<j)&&(nm_bit.get(j*nCount+i)==true))
	    	{
	    	   if ((i>=inCount && i<=inOutCount)||   // output -> ???
	    	       (j<inCount && i<inCount))        // input -> input
	    	      nm_bit.clear(j*nCount+i);
	    	}
	    	if ((i!=j)&&(nm_bit.get(j*nCount+i)==true)&&(nm_bit.get(i*nCount+j)==true))
	    	{
	    	    nm_bit.clear(i*nCount+j);
	    	}
	    	if (i!=j)
	    	{
	    	   if ((nm_bit.get(j*nCount+j)==false)||(nm_bit.get(i*nCount+i)==false))
	    	   {
	    	      nm_bit.clear(j*nCount+i);
	    	      nm_bit.clear(i*nCount+j);
	    	   }	    	      
	    	}
	    } 
	    
   	nmPreset();
   
        cop=0;
    	// crossing over wm_bit
    	for (int z=0; z<a.wm_bit.size(); z++)
    	{
    	    if ((cop<c.crossover_type)&&(Math.abs(rand.nextFloat())<c.ann_crossover_rate))
            {
		cop++;
    	    	cs=1-cs;
	    }
    		
    	    if (cs==0)
    	    {
    	    	if (b.wm_bit.get(z)==true)
    	    	    wm_bit.set(z);
    	    	else
    	    	    wm_bit.clear(z);
    	    }
    	    else
    	    {
    	    	if (a.wm_bit.get(z)==true)
    	    	    wm_bit.set(z);
    	    	else
    	    	    wm_bit.clear(z);
    	    }
    	}
    	
    	// TODO: don't cross over spline_matrix -> next version
    	
    	if (splineNodeCount>0)
    	  for (int z=0; z<a.sm_bit.size(); z++)
          {
    	       if (a.sm_bit.get(z)==true)
    	           sm_bit.set(z);
    	       else
    	           sm_bit.clear(z);	  
    	  }
    	
    	cop=0;
    	// crossing over sm_bit
    	if (splineNodeCount>0)
    	for (int z=0; z<a.sm_bit.size(); z++)
    	{
    	    if ((cop<c.crossover_type)&&(Math.abs(rand.nextFloat())<c.cs_crossover_rate))
	    {
	        cop++;
    	    	cs=1-cs;
	    }
    		
    	    if (cs==0)
    	    {
    	    	if (b.sm_bit.get(z)==true)
    	    	    sm_bit.set(z);
    	    	else
    	    	    sm_bit.clear(z);
    	    }
    	    else
    	    {
    	    	if (a.sm_bit.get(z)==true)
    	    	    sm_bit.set(z);
    	    	else
    	    	    sm_bit.clear(z);
    	    }
    	}
    	
    	if (splineNodeCount>0)
    	for (int z=0; z<10-1; z++)
    	{
    	     // TODO: splines' x-values are mixed up
    	     if (getFloatOuttaSm(z*2)>getFloatOuttaSm((z+1)*2))
    	     	crippled++;
    	}
    	
    	cop=0;
    	// crossing over ss_bit
    	for (int z=0; z<a.ss_bit.size(); z++)
    	{
    	    if ((cop<c.crossover_type)&&(Math.abs(rand.nextFloat())<c.ann_crossover_rate))
            {
		cop++;
    	    	cs=1-cs;
	    }
    		
    	    if (cs==0)
    	    {
    	    	if (b.ss_bit.get(z)==true)
    	    	    ss_bit.set(z);
    	    	else
    	    	    ss_bit.clear(z);
    	    }
    	    else
    	    {
    	    	if (a.ss_bit.get(z)==true)
    	    	    ss_bit.set(z);
    	    	else
    	    	    ss_bit.clear(z);
    	    }
    	}    	
    	
    	cop=0;
    	// crossing over mm_bit
    	for (int z=0; z<a.mm_bit.size(); z++)
    	{
    	    if ((cop<c.crossover_type)&&(Math.abs(rand.nextFloat())<c.ann_crossover_rate))
            {
		cop++;
    	    	cs=1-cs;
	    }
    		
    	    if (cs==0)
    	    {
    	    	if (b.mm_bit.get(z)==true)
    	    	    mm_bit.set(z);
    	    	else
    	    	    mm_bit.clear(z);
    	    }
    	    else
    	    {
    	    	if (a.mm_bit.get(z)==true)
    	    	    mm_bit.set(z);
    	    	else
    	    	    mm_bit.clear(z);
    	    }
    	}
    }

    // mutation of the genotype (mitosis)
    public void mutate()
    {
    	// let the mutation begin :)
    	// first we mutate the wm_bit matrix    	
    	for (int z=0; z<nm_bit.size(); z++)
    	{
    	    if (Math.abs(rand.nextFloat())<c.ann_mutation_rate)
    	    {
    	    	if (nm_bit.get(z)==true)
    	    	   nm_bit.clear(z);
    	    	else
    	    	   nm_bit.set(z);
    	    }
    	}
	    	
    	// mutate the  csnActive_bit matrix    	
	if (c.spline_node_count>0)
    	for (int z=0; z<csnActive_bit.size(); z++)
    	{
    	    if (Math.abs(rand.nextFloat())<c.cs_mutation_rate)
    	    {
    	    	if (csnActive_bit.get(z)==true)
    	    	   csnActive_bit.clear(z);
    	    	else
    	    	   csnActive_bit.set(z);
    	    }
    	}
	
	// mutate the  ss_bit matrix    	
	for (int z=0; z<ss_bit.size(); z++)
    	{
    	    if (Math.abs(rand.nextFloat())<c.ann_mutation_rate)
    	    {
    	    	if (ss_bit.get(z)==true)
    	    	   ss_bit.clear(z);
    	    	else
    	    	   ss_bit.set(z);
    	    }
    	}	
    	
    	// mutate the mm_bit matrix    	
	for (int z=0; z<mm_bit.size(); z++)
    	{
    	    if (Math.abs(rand.nextFloat())<c.ann_mutation_rate)
    	    {
    	    	if (mm_bit.get(z)==true)
    	    	   mm_bit.clear(z);
    	    	else
    	    	   mm_bit.set(z);
    	    }
    	}	
	   	    	
    	// validate the nm_bit matrix
	for (int i=0; i<nCount; i++)
	    for (int j=0; j<nCount; j++)
	    {
	    	if ((i>j)&&(nm_bit.get(j*nCount+i)==true))
	    	{
	    	   if ((i>=inOutCount && j<inCount)||    // hidden -> input
	    	       (i>=inCount && i<inOutCount)||   // output -> ???
	    	       (j<inCount && i<inCount)||       // input -> input
	    	       (i>=inOutCount && j>=inOutCount))	// hidden -> hidden (src>target)   	        
	    	       
	    	      nm_bit.clear(j*nCount+i);
	    	}
	    	if ((i<j)&&(nm_bit.get(j*nCount+i)==true))
	    	{
	    	   if ((i>=inCount && i<=inOutCount)||   // output -> ???
	    	       (j<inCount && i<inCount))        // input -> input
	    	      nm_bit.clear(j*nCount+i);
	    	}
	    	if ((i!=j)&&(nm_bit.get(j*nCount+i)==true)&&(nm_bit.get(i*nCount+j)==true))
	    	{
	    	    nm_bit.clear(i*nCount+j);
	    	}
	    	if (i!=j)
	    	{
	    	   if ((nm_bit.get(j*nCount+j)==false)||(nm_bit.get(i*nCount+i)==false))
	    	   {
	    	      nm_bit.clear(j*nCount+i);
	    	      nm_bit.clear(i*nCount+j);
	    	   }	    	      
	    	}
	    } 
	nmPreset();
		
	// now let's go to weight_matrix mutation    	
	for (int z=0; z<wm_bit.size(); z++)
    	{
    	    if (Math.abs(rand.nextFloat())<c.ann_mutation_rate)
    	    {
    	    	if (wm_bit.get(z)==true)
    	    	   wm_bit.clear(z);
    	    	else
    	    	   wm_bit.set(z);
    	    }
    	}
    	
    	// check for validity of IEEE numbers
    	for (int z=0; z<nCount*nCount; z++)
    	{
    	    // first make all signed values to unsigned ones
    	    if (wm_bit.get(z*32)==true)
    	    	wm_bit.clear(z*32);
    	    
    	    // check if floats are valid	
    	    if (getFloatOuttaWm(z)==Float.NaN)
    	    {
    	    	crippled++;
    	    }
    	}
	
    	// and last, mutate the spline matrix
    	if (splineNodeCount>0)
    	for (int z=0; z<sm_bit.size(); z++)
    	{
    	    if (Math.abs(rand.nextFloat())<c.cs_mutation_rate)
    	    {
    	    	if (sm_bit.get(z)==true)
    	    	   sm_bit.clear(z);
    	    	else
    	    	   sm_bit.set(z);
    	    }
    	}
    	
    	// and check out if the splines are "good"
    	if (splineNodeCount>0)
    	for (int z=0; z<10; z++)
    	{	
    	    // check if floats are valid 	
    	    if (getFloatOuttaSm(z*2)==Float.NaN)
    	    {
    	    	// TODO: decide how we should handle this
    	    	crippled++;
    	    }
    	}
    	
    	// check the splines for correct x-values (x1<x2<...<xn)
    	if (splineNodeCount>0)
    	for (int z=0; z<10-1; z++)
    	{
    	    if (getFloatOuttaSm(z*2)>getFloatOuttaSm((z+1)*2))
    	    {
    	    	// got a bad one
    	    	// TODO: decide how we should handle this
    	    	crippled++;
    	    }
    	}
    }

    // convert a bitset (32 bit length) to an IEEE float
    public float IEEE2float(BitSet f)
    {
    	int z=0;
    	float sign=0.0f;    	
    	float exponent=0.0f;
    	float mantissa=0.0f;
    	
    	// check for signed float
    	if (f.get(31)==true)
    	   sign=-1.0f;
    	else
           sign=1.0f; 		
           
        // extract exponent out of bitstring
        for (z=1; z<9; z++)
        {
           float digit=0.0f;
           if (f.get(31-z)==true)
               digit=1.0f; 	
           exponent=exponent+digit*(float)Math.pow((double)2,(double)(8-z));
        }
        
        if (exponent==255.0f)
            return Float.NaN;
        
        exponent=exponent-127.0f;
        
        // extract mantissa out of bitstring
        for (z=9; z<32; z++)
        {
           float digit=0.0f;
           if (f.get(31-z)==true)
               digit=1.0f;
           mantissa=mantissa+digit*1.0f/((float)Math.pow((double)2,(double)(z-8)));    
        }
        mantissa=mantissa+1;
        
        // put all the stuff together
        return sign*mantissa*(float)Math.pow((double)2,(double)exponent);
    }

    // get a bitset out of a 32 bit IEEE float
    public BitSet float2IEEE(float f)
    {
    	BitSet b=new BitSet(32);
    	    	    	
    	// get integer corresponding to our 32 bit IEEE bitstring 
    	int ln=Float.floatToIntBits(f);
    	
    	int z=0;
    	// convert to bitstring
    	while (ln>0)
    	{
    	   if (ln%2==1)
    	       b.set(z);
	   ln=ln/2;   
	   z++; 	    	    
    	}
    	 
    	return b;   	
    }
    
    // get a single weight value out of the weight_matrix bitstring
    public float getFloatOuttaWm(int idx)
    {
    	BitSet tmp=new BitSet(32);
    	for (int z=0; z<32; z++)
    	     if (wm_bit.get(idx*32+z)==true)
    	     	tmp.set(z);
    	     else
    	     	tmp.clear(z);
    	     	
    	return IEEE2float(tmp);
    }

    // get a single spline (x/y) value out of the spline_matrix bitstring 
    public float getFloatOuttaSm(int idx)
    {
    	BitSet tmp=new BitSet(32);
    	for (int z=0; z<32; z++)
    	     if (sm_bit.get(idx*32+z)==true)
    	     	tmp.set(z);
    	     else
    	     	tmp.clear(z);
    	     	
    	return IEEE2float(tmp);
    }
    
    // store a value into weight_matrix at position idx
    public void putFloatIntoWm(int idx, float f)
    {
    	BitSet tmp=float2IEEE(f);
    	for (int z=0; z<32; z++)
    	{
    	     if (tmp.get(z)==true)
    	     	wm_bit.set(32*idx+z);
    	     else
    	     	wm_bit.clear(32*idx+z);
    	}
    }
    
    // store a value into spline_matrix at position idx
    public void putFloatIntoSm(int idx, float f)
    {
    	BitSet tmp=float2IEEE(f);
    	for (int z=0; z<32; z++)
    	{
    	     if (tmp.get(z)==true)
    	     	sm_bit.set(32*idx+z);
    	     else
    	     	sm_bit.clear(32*idx+z);
    	}
    }
        
    // initalize the weight_matrix with random values
    public void initRandomWm()
    {
	for (int z=0; z<nCount*nCount; z++)
	    putFloatIntoWm(z,Math.abs(rand.nextFloat()));
    }
    
    // this function is used to active all input/output neurons
    public void nmPreset()
    {
    	for (int z=0; z<inOutCount; z++)
    	    nm_bit.set(z*nCount+z);
    }
    
    public void makeInputsConnected()
    {
    	for (int i=0; i<inCount; i++)
    	    for (int j=inOutCount; j<nCount; j++)
    	    {
    	    	nm_bit.set(j*nCount+i);
    	    }
    }
    
    public void makeOutputsConnected()
    {
    	for (int i=0; i<inCount; i++)
    	    for (int j=inCount; j<inOutCount; j++)
    	    {
    	    	nm_bit.set(j*nCount+i);
    	    }
    	for (int i=inOutCount; i<nCount; i++)
    	    for (int j=inCount; j<inOutCount; j++)
    	    {
    	    	nm_bit.set(j*nCount+i);
    	    }
    }
    
    public void initRandomCsn()
    {
	int tmp=0;
    	for (int z=0; z<splineNodeCount*nCount; z++)
	    if (Math.abs(rand.nextFloat())>0.5f)
	    {
		csnActive_bit.set(z);
		tmp++;
	    }
            else
                csnActive_bit.clear(z);
	
	// make sure that there are at least 3 active cs-nodes
	correctCsn();
        //System.out.println("INITRANDOMCSN: tmp="+tmp);
    }

    public void correctCsn()
    {
    	// make sure that for each neuron at least 3 cs-nodes are active
	for (int nIdx=0; nIdx<nCount; nIdx++)
	{	    	
	    int tmp=getCsnCount(nIdx);
	    if (tmp<3)
	    {
	    	/*
	    	System.out.println("GENOTYPE: Trying to correct csnActive_bit");
	    	System.out.println("GENOTYPE: csnCount="+tmp);
	    	*/
	    	for (int z2=tmp; z2<3; z2++)
	    	{
	    	    //System.out.println("GENOTYPE: z2="+z2);
		    int p=Math.abs(rand.nextInt())%splineNodeCount;
		    //System.out.println("GENOTYPE: checking p="+p);
		    while (csnActive_bit.get(p+nIdx*splineNodeCount))
		    {
		    	p=Math.abs(rand.nextInt())%splineNodeCount;
		        //System.out.println("GENOTYPE: checking p="+p);
		    }
		    //System.out.println("GENOTYPE: activating p="+p);
		    csnActive_bit.set(p+nIdx*splineNodeCount);
	    	}
	    	//System.out.println("GENOTYPE: csnActive_bit corrected = "+getCsnCount());
	    }
	}
    }
    
    public void initRandomSS()
    {
 	for (int z=0; z<ss_bit.size(); z++)
    	    if (Math.abs(rand.nextFloat())>0.5f)
		ss_bit.set(z);
            else
                ss_bit.clear(z);
    }
    
    public void initRandomMM()
    {
 	for (int z=0; z<mm_bit.size(); z++)
    	    if (Math.abs(rand.nextFloat())>0.5f)
		mm_bit.set(z);
            else
                mm_bit.clear(z);
    }    
    
    // initialize the neuron_matrix (MMM) with random value and check them for correctness
    public void initRandomNm()
    {
	for (int z=0; z<nCount*nCount; z++)
	    if (Math.abs(rand.nextFloat())>0.5f)
		nm_bit.set(z);
            else
                nm_bit.clear(z);
 		
	// validate the matrix
	for (int i=0; i<nCount; i++)
	    for (int j=0; j<nCount; j++)
	    {
	    	if ((i>j)&&(nm_bit.get(j*nCount+i)==true))
	    	{
	    	   if ((i>=inOutCount && j<inCount)||    // hidden -> input
	    	       (i>=inCount && i<inOutCount)||   // output -> ???
	    	       (j<inCount && i<inCount)||       // input -> input
	    	       (i>=inOutCount && j>=inOutCount))	// hidden -> hidden (src>target)   	        
	    	       
	    	      nm_bit.clear(j*nCount+i);
	    	}
	    	if ((i<j)&&(nm_bit.get(j*nCount+i)==true))
	    	{
	    	   if ((i>=inCount && i<inOutCount)||   // output -> ???
	    	       (j<inCount && i<inCount))        // input -> input
	    	      nm_bit.clear(j*nCount+i);
	    	}
	    	// a->b and b->a
	    	if ((i!=j)&&(nm_bit.get(j*nCount+i)==true)&&(nm_bit.get(i*nCount+j)==true))
	    	{
	    	    nm_bit.clear(i*nCount+j);
	    	}
	    	// if neuron i or neuron j is inactive then it's not possible to have them
	    	// connected
	    	if (i!=j)
	    	{
	    	   if ((nm_bit.get(j*nCount+j)==false)||(nm_bit.get(i*nCount+i)==false))
	    	   {
	    	      nm_bit.clear(j*nCount+i);
	    	      nm_bit.clear(i*nCount+j);
	    	   }	    	      
	    	}
	    } 
	
	nmPreset();
	/*
	makeInputsConnected();
	*/
	//makeOutputsConnected();
    }
    
    // initialize the spline_matrix with random values
    public void initRandomSm()
    {
	for (int z=0; z<splineNodeCount*nCount; z++)
	{
	    // TODO: den Step von 20 noch mal anschauen
	    putFloatIntoSm(z*2,(float)(z*20));
	    putFloatIntoSm(z*2+1,Math.abs(rand.nextFloat()));
	}		
    }
    
    /*
    // show the bitstring for a IEEE float
    public void showIEEE32(BitSet b)
    {
    	for (int z=0; z<32; z++)
    	    if (b.get(31-z)==true)
    	    	System.out.print("1");
    	    else
    	    	System.out.print("0");
    	System.out.println("");    	     	 
    }
    
    // pretty much the same as showIEEE32
    public void showIEEE(BitSet f)
    {
    	System.out.println(f.toString());
    }
    */
}
