package algae.simulation.agents;

import java.util.*;
import java.lang.*;
import java.lang.Object;
import java.lang.Math;
import java.lang.ArrayIndexOutOfBoundsException;

public class PhenoType {
	
    public int nCount = 50; // count of neurons used
	    
    public int inCount = 17; // number of input-neurons
    public int outCount = 5; // number of output-neurons

    public int inOutCount = inCount+outCount;
    public int hiddenCount = nCount-inOutCount;
    
    public int splineNodeCount = 10;

    public int sensitivity[];
    public int metabolic[];

    // neurons from 1 - 17 (resp. 0-16) = input neurons	
    // neurons from 18 - 22 (resp. 17-21) = output neurons
    // rest of it = hidden layer neurons
    public byte neuron_matrix[][]; // [from][to]
    public float weight_matrix[][];
    public float spline_matrix[][];
    public byte csn_active_matrix[];
    public byte neuron_transferFunc[];

    public float neuronOutput[];
    public boolean visited[];
    
    public CubicSpline cs[];
    public Vector x;
    public Vector y;
    
    public float phenoANNBias=0.0f;

    public int funcSel=-1;
     
    public int getConnCount()
    {
    	int connCount=0;
    	for (int i=0; i<nCount; i++)
    	{
    	    for (int j=0; j<nCount; j++)
    	    {
    	    	if (connected(i,j))
    	    	   connCount++;
    	    }
    	}
    	return connCount;
    } 
    
    public int getChildNodeCount(int i)
    {
    	int cnc=0;
    	for (int j=0; j<nCount; j++)
    	    if (connected(i,j))
    	       cnc++;
    	return cnc;
    }
    
    public int getParentNodeCount(int j)
    {
    	int pnc=0;
    	for (int i=0; i<nCount; i++)
    	    if (connected(i,j))
    	       pnc++;
    	return pnc;
    }
    
    public int getActiveNeuronCount()
    {
    	int ncnt=0;
    	for (int i=0; i<nCount; i++)
    	    if (neuron_matrix[i][i]==1)
    	       ncnt++;
    	return ncnt;
    }
    
    public boolean isActive(int i)
    {
    	return (neuron_matrix[i][i]==1);
    }
    
    // get active number for nth node
    public int getNth2Ac(int n)
    {
    	if (n==0) return 1;
    	
    	int ac=0;
    	    	
    	for (int z=0; z<n; z++)
    	{
    	    if (isActive(z))
    	       ac++;
    	}
    	
    	return ac;
    }
        
    public PhenoType(GenoType g){
	//System.out.println("PHENOTYPE.constructor: begin");
	nCount=g.nCount;
	inCount=g.inCount;
	outCount=g.outCount;
	inOutCount=g.inOutCount;
	hiddenCount=g.hiddenCount;
	splineNodeCount=g.splineNodeCount;
	funcSel=g.funcSel;
	phenoANNBias=g.genoANNBias;
		
	neuron_matrix = new byte[nCount][nCount]; // [from][to]
    	weight_matrix = new float[nCount][nCount];
    	
    	if (splineNodeCount>0)
    	{
    	    spline_matrix = new float[2][splineNodeCount*nCount];
 	    csn_active_matrix = new byte[splineNodeCount*nCount];
	}

	metabolic=new int[2]; // 0=light 1=substrate 
	for (int z=0; z<2; z++)
	    metabolic[z]=(g.mm_bit.get(z)?1:0);
	
	sensitivity=new int[4];
	// 0=b 1=p 2=s 3=l	
	for (int z=0; z<4; z++)
	    sensitivity[z]=(g.ss_bit.get(z)?1:0);
	
    	neuronOutput = new float[nCount];
    	visited = new boolean[nCount];    
	
	neuron_transferFunc = new byte[nCount];
	
    	// extract neuron matrix (miller matrix) out of genotype
    	for (int i=0; i<nCount; i++)
    	    for (int j=0; j<nCount; j++)
    	    	if (g.nm_bit.get(j*nCount+i)==true)
    	    	    neuron_matrix[i][j]=1;
    	    	else
    	    	    neuron_matrix[i][j]=0;    	    		

	//printNeuronMatrix();
    	
    	// extract weight matrix out of genotype   
    	for (int i=0; i<nCount; i++)
    	    for (int j=0; j<nCount; j++)
    	    	weight_matrix[i][j]=g.getFloatOuttaWm(j*nCount+i);
    	
    	if (splineNodeCount<1)
    	   return;
    	    	                         	
    	// extract splines out of genotype
    	for (int i=0; i<splineNodeCount*nCount; i++)
    	{
    	    spline_matrix[0][i]=g.getFloatOuttaSm(i*2);
    	    spline_matrix[1][i]=g.getFloatOuttaSm(i*2+1);
    	}
    	    
    	cs=new CubicSpline[nCount];
    	for (int nIdx=0; nIdx<nCount; nIdx++)
    	{
    	int csnc=csnc=g.getCsnCount(nIdx);
	
    	Vector x=new Vector(csnc); 
    	Vector y=new Vector(csnc);    	

	//System.out.println("PHENOTYPE: csnc="+csnc);
    	// construct vectors used by splines
    	int tmp=0;
    	for (int z=0; z<splineNodeCount; z++)
    	{
    	    if (g.isCsnBit(z+nIdx*splineNodeCount)==true)
    	    {
    		//System.out.println("PHENOTYPE: adding coord for csn "+tmp);
    	    	x.add(tmp,new Double(spline_matrix[0][z+nIdx*splineNodeCount]));
    	    	y.add(tmp,new Double(spline_matrix[1][z+nIdx*splineNodeCount]));
    	    	tmp++;
    	    }
    	}
   	/*System.out.println("PHENOTYPE.constructor: csnc="+csnc);
	System.out.println("splineNodeCount="+splineNodeCount);
	System.out.println("nCount="+nCount);
	*/
    	cs[nIdx]=new CubicSpline(csnc,x,y);
	//System.out.println("PHENOTYPE.constructor: end");
	}
    }
     
    public float applyBias(float f)
    {
    	float dummyFloat=f-phenoANNBias;
    	return (dummyFloat<0?0:dummyFloat);
    }
        
    public boolean connected(int i, int j)
    {
    	return ((neuron_matrix[i][j]==1)&&(i!=j));
    }    
                
    public float transferFunc(int nIdx, int dummyFuncSel, float val)
    {
    	// TODO: if val is out of bounds simply return most outter value inbounds
	float res=0.0f;
	if (splineNodeCount>0)
	{
	   try
  	   {
	      res=(float)cs[nIdx].getValue(Math.abs((double)val));
	   } catch (ArrayIndexOutOfBoundsException e)
	   {
	      //System.out.println("EXCEPTION: Value == "+val);
	      res=0.0f;
	   }
	}
	else
	switch (funcSel)
	{
	   //String []transferFuncs={"tan","sin","cos","atan","asin","acos"};	   
           case 1: res=(float)Math.tan((double)val); break;
	   case 2: res=(float)Math.sin((double)val); break;
	   case 3: res=(float)Math.cos((double)val); break;
	   case 4: res=(float)Math.atan((double)val); break;
	   case 5: res=(float)Math.asin((double)val); break;
	   case 6: res=(float)Math.acos((double)val); break;
	}
		
	return res;
    }
    
    // get number of parent nodes
    public int getPrevNum(int n)
    {
    	int cnt=0;
    	for (int z=0; z<nCount; z++)
    	    if ((neuron_matrix[z][n]==1)&&(z!=n))
    	    	cnt++;
    	return cnt;
    }
    
    // get nth parent node 
    public int getPrev(int i, int n)
    {
    	int nth=0;
    	for (int z=0; z<nCount; z++)
    	    if ((connected(z,i))/*&&(!visited[z])*/)
    	    {
    	    	nth++;
    	    	if (nth==n)
    	    	   return z;    	    	
    	    }
    	return -1;
    }
    
    // calculate output for neuron n
    public float calculateNeuron(int n)
    {
    	float result=0.0f;
    	int prevNum = getPrevNum(n);
    	
    	for (int i=0; i<nCount; i++)
    	     visited[i]=false;

	float input=0.0f;

    	for (int z=0; z<prevNum; z++)
    	{ 
    	     int prev=getPrev(n,z+1);
    	     input+=neuronOutput[prev]*weight_matrix[prev][n];
    	     visited[prev]=true;
    	}
    	if (input==0.0f) 
    	   return 0.0f;
    	else
    	   return transferFunc(n,111,applyBias(input));   	
    }
    
    public void normalizeInput()
    {
    	float maxVal=0.0f;
    	
    	for (int z=0; z<inCount; z++)
            if (neuronOutput[z]>maxVal)
            	maxVal=neuronOutput[z];
        
        // avoid division by zero    	
        if (maxVal==0.0f) return;
        
        for (int z=0; z<inCount; z++)
            neuronOutput[z]/=maxVal;                
    }
    
    // process current neural network
    // and return the number of the winner neuron (the winner takes all)
    public int processNet(float inVec[])
    {
    	int i=0,j=0,z=0;
    	
    	// fill input vector with input data (2BDone correctly)
    	for (z=0; z<inCount; z++)
            neuronOutput[z]=inVec[z];    	     
        
        // reset other outputs 
        for (z=inCount; z<nCount; z++)
            neuronOutput[z]=0.0f;    

	// apply sensitivity
	for (z=1; z<inCount; z++)
	    neuronOutput[z]*=sensitivity[(z-1)/4];
            
        normalizeInput();
        
        // calculate all outputs for neurons from top down
        for (z=inOutCount; z<nCount; z++)
        {
            if (neuron_matrix[z][z]!=0) // is neuron active ?
            {
            	neuronOutput[z]=calculateNeuron(z);
            }
        }

	int max=0;  
	// calculate outputs and look out for winner neuron too      
        for (z=inCount; z<inOutCount; z++)
        {
             neuronOutput[z]=calculateNeuron(z);
             if (neuronOutput[z]>neuronOutput[max])
             	max=z;
        }
        
        if (max==0) 
           return 0;             	
        else
    	   return (max-inCount);
    }
    
    public void printNeuronMatrix()
    {
    	for (int i=0; i<nCount; i++)
    	{
    	    for (int j=0; j<nCount; j++)
    	    {
    	    	if (neuron_matrix[i][j]!=1)
    	    	    System.out.print(" ");
    	    	else
    	    	    System.out.print(neuron_matrix[i][j]);
    	    }
    	    System.out.print("\n");	
    	}    	
    }
    
    public void printWeightMatrix()
    {
    	for (int i=0; i<nCount; i++)
    	{
    	    for (int j=0; j<nCount; j++)
    	    {
    	    	System.out.print(weight_matrix[i][j]);
    	    }
    	    System.out.print("\n");	
    	}    	
    }
    
    public void printOutput()
    {
    	for (int z=inCount; z<inOutCount; z++)
    	{
    	    System.out.println("OutputNeuron #"+(z-inCount+1)+" = "+neuronOutput[z]);
    	}
    }
}
