package algae.xmlstuff;

import algae.simulation.Settings;

import java.io.*;
import java.util.*;
import java.util.Date;
import java.text.DateFormat;
import com.sun.xml.parser.Resolver;
import com.sun.xml.tree.*;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.w3c.dom.*;

public class SettingsExport {
	
	Settings c;
	
	public void exportSettingsFile(File xmlfile, Settings cfg)
	{
	    c=cfg;       
            
	    XmlDocument xmldoc=new XmlDocument(); 
       	
       	    ElementNode xmldata = (ElementNode) xmldoc.createElement ("Settingsblock");
	
	    Date today = new Date();
	    DateFormat df;
	
	    xmldata.setAttribute ("version","v1.3");
            xmldata.setAttribute ("creator","ALGAE RC9+");
            xmldata.setAttribute ("name",xmlfile.getName());
            xmldata.setAttribute ("date",DateFormat.getDateTimeInstance().format(today));  
                
            ExportSimulationSettings(xmldoc,xmldata);
            ExportLightSettings(xmldoc,xmldata);
            ExportSubstrateSettings(xmldoc,xmldata);
            ExportAlgaesSettings(xmldoc,xmldata);
            ExportSensorsSettings(xmldoc,xmldata);
            ExportActionsSettings(xmldoc,xmldata);
            ExportANNSettings(xmldoc,xmldata);
            ExportEvolutionSettings(xmldoc,xmldata);

	    // Write the stuff previously created
            try  
            {
                FileOutputStream ostream  = new FileOutputStream(xmlfile);
                OutputStreamWriter owrite = new OutputStreamWriter(ostream);
                owrite.write("<?xml version=\"1.0\"?>\n\r");
                owrite.write("<!DOCTYPE Element_Definitions SYSTEM \"FFv1_3.dtd\">\n\r");
                xmldata.write(owrite);
                owrite.flush();
                ostream.close();
            }
            catch (IOException ex) 
            {
                System.out.println("ERROR at Export: IOException writing to " + xmlfile.getAbsolutePath());
            }
	}
	
    public void ExportSimulationSettings(XmlDocument xmldoc, ElementNode XML)
    {
    	ElementNode ne = (ElementNode) xmldoc.createElement("Simulation");
    	XML.appendChild(ne);
    	
    	ne.setAttribute("duration",Integer.toString(c.duration));
    	ne.setAttribute("width",Integer.toString(c.width));
    	ne.setAttribute("height",Integer.toString(c.height));
    	
    	ne.setAttribute("randseed",Integer.toString(c.randSeed));
    	    	
    	ne.setAttribute("pheromone_dec_rate",Float.toString(c.pheromone_dec_rate));
    }
    
    public void ExportAlgaesSettings(XmlDocument xmldoc, ElementNode XML)
    {
    	ElementNode ne = (ElementNode) xmldoc.createElement("Algaes");
    	XML.appendChild(ne);
    	
    	ne.setAttribute("initial_algaes",Integer.toString(c.initial_algaes));
    	ne.setAttribute("initial_energy",Float.toString(c.initial_energy));
    	ne.setAttribute("agent_transluency",Float.toString(c.agent_transluency));
    	ne.setAttribute("pheromone_radius",Float.toString(c.pheromone_radius));
    }
    
    public void ExportSensorsSettings(XmlDocument xmldoc, ElementNode XML)
    {
    	ElementNode ne = (ElementNode) xmldoc.createElement("Sensors");
    	XML.appendChild(ne);
    	
    	ne.setAttribute("pheromone_fov",Integer.toString(c.pheromone_fov));
    	ne.setAttribute("pheromone_range",Integer.toString(c.pheromone_range));
    	
    	ne.setAttribute("substrate_fov",Integer.toString(c.substrate_fov));
    	ne.setAttribute("substrate_range",Integer.toString(c.substrate_range));
    	
    	ne.setAttribute("light_fov",Integer.toString(c.light_fov));
    	ne.setAttribute("light_range",Integer.toString(c.light_range));
    	
    	ne.setAttribute("bumper_range",Integer.toString(c.bumper_range));
    }
        
    public void ExportActionsSettings(XmlDocument xmldoc, ElementNode XML)
    {
    	ElementNode ne = (ElementNode) xmldoc.createElement("Actions");
    	XML.appendChild(ne);
    
	ne.setAttribute("energy_movement",Float.toString(c.energy_movement));
	ne.setAttribute("energy_mitose",Float.toString(c.energy_movement));
	ne.setAttribute("energy_meiose",Float.toString(c.energy_meiose));    
    	ne.setAttribute("energy_pheromone_output",Float.toString(c.energy_pheromone_output));
	ne.setAttribute("energy_substrate",Float.toString(c.energy_substrate));
    	ne.setAttribute("energy_light",Float.toString(c.energy_light));
    	ne.setAttribute("energy_standstill",Float.toString(c.energy_standstill));
    	
    	ne.setAttribute("min_energy_mitose",Float.toString(c.min_energy_mitose));
    	ne.setAttribute("min_energy_meiose",Float.toString(c.min_energy_meiose));
    	ne.setAttribute("min_energy_pheromone_output",Float.toString(c.min_energy_pheromone_output));
    	
    	ne.setAttribute("max_pheromone_saturation",Float.toString(c.max_pheromone_saturation));
    	ne.setAttribute("max_substrate_saturation",Float.toString(c.max_substrate_saturation));
    }
    
    public void ExportANNSettings(XmlDocument xmldoc, ElementNode XML)
    {
    	ElementNode ne = (ElementNode) xmldoc.createElement("ANN");
    	XML.appendChild(ne);
    
	ne.setAttribute("number_of_neurons",Integer.toString(c.number_of_neurons));
	ne.setAttribute("initial_ann_bias",Float.toString(c.initial_ann_bias));
	
	if (c.spline_node_count!=-1)
	{
	   ne.setAttribute("cubic_spline_nodecount",Integer.toString(c.spline_node_count));
	   ne.setAttribute("ann_transfer_func","-1");
	}
	else
	{
	    ne.setAttribute("cubic_spline_nodecount","-1");
	    ne.setAttribute("ann_transfer_func",Integer.toString(c.ann_transfer_func));
	}
    }    
     
    public void ExportEvolutionSettings(XmlDocument xmldoc, ElementNode XML)
    {
    	ElementNode ne = (ElementNode) xmldoc.createElement("Evolution");
    	XML.appendChild(ne);

	// evolution settings
	ne.setAttribute("crossover_type",Integer.toString(c.crossover_type));
	
	ne.setAttribute("ann_mutation_rate",Float.toString(c.ann_mutation_rate));
        ne.setAttribute("ann_crossover_rate",Float.toString(c.ann_crossover_rate));  	
	ne.setAttribute("cs_mutation_rate",Float.toString(c.cs_mutation_rate));
        ne.setAttribute("cs_crossover_rate",Float.toString(c.cs_crossover_rate));  	
    }

    public void ExportLightSettings(XmlDocument xmldoc, ElementNode XML)
    {
    	ElementNode ne = (ElementNode) xmldoc.createElement("Light");
    	XML.appendChild(ne);
    
    	switch (c.light_enabled)
    	{
    	    case 0: ne.setAttribute("light_enabled","never"); break;
	    case 1: ne.setAttribute("light_enabled","all the time"); break;
    	    case 2: ne.setAttribute("light_enabled","function"); break;
    	    default: ne.setAttribute("light_enabled",""); break;
    	}
    	    	
	ne.setAttribute("light_intensity",Float.toString(c.light_intensity));    	    	

	switch (c.light_cycle_function)
	{
	    case 1: ne.setAttribute("light_cycle_func","sin(t)"); break;
	    case 2: ne.setAttribute("light_cycle_func","rectangle(t)"); break;
	    case 3: ne.setAttribute("light_cycle_func","chainsaw(t)"); break;
	    case 4: ne.setAttribute("light_cycle_func","zigzag(t)"); break;
	    default: ne.setAttribute("light_cycle_func",""); break;  
	}
	
	ne.setAttribute("light_distribution_func","1/y");
    }
    
    public void ExportSubstrateSettings(XmlDocument xmldoc, ElementNode XML)
    {
	ElementNode ne = (ElementNode) xmldoc.createElement("Substrate");
    	XML.appendChild(ne);
    	
    	ne.setAttribute("substrate_amount",Integer.toString(c.substrate_amount));
	ne.setAttribute("substrate_renew_rate",Float.toString(c.substrate_renew_rate));
	ne.setAttribute("substrate_for_dead_agent",Float.toString(c.substrate_for_dead_agent));
	ne.setAttribute("periodic_substrate_amount",Integer.toString(c.substrate_periodic_amount));
	ne.setAttribute("periodic_substrate_steps",Integer.toString(c.substrate_periodic_steps));
	
	// substrate_function: TODO
	ne.setAttribute("periodic_substrate",Integer.toString(c.periodic_substrate));
    }
    
}